@@ -3,9 +3,11 @@
 use crate::soft::{x2, x4};
 use crate::types::*;
 use core::ops::*;
+use zerocopy::{AsBytes, FromBytes};
+use zerocopy_derive::{AsBytes, FromBytes, FromZeroes};
 
 #[repr(C)]
-#[derive(Clone, Copy)]
+#[derive(Clone, Copy, FromBytes, AsBytes, FromZeroes)]
 pub union vec128_storage {
     d: [u32; 4],
     q: [u64; 2],
@@ -452,11 +454,14 @@ impl Machine for GenericMachine {
     }
 }
 
-#[derive(Copy, Clone, Debug, PartialEq)]
+#[derive(Copy, Clone, Debug, PartialEq, FromBytes, AsBytes, FromZeroes)]
+#[repr(transparent)]
 pub struct u32x4_generic([u32; 4]);
-#[derive(Copy, Clone, Debug, PartialEq)]
+#[derive(Copy, Clone, Debug, PartialEq, FromBytes, AsBytes, FromZeroes)]
+#[repr(transparent)]
 pub struct u64x2_generic([u64; 2]);
-#[derive(Copy, Clone, Debug, PartialEq)]
+#[derive(Copy, Clone, Debug, PartialEq, FromBytes, AsBytes, FromZeroes)]
+#[repr(transparent)]
 pub struct u128x1_generic([u128; 1]);
 
 impl From<u32x4_generic> for vec128_storage {
@@ -561,53 +566,45 @@ impl BSwap for u128x1_generic {
 impl StoreBytes for u32x4_generic {
     #[inline(always)]
     unsafe fn unsafe_read_le(input: &[u8]) -> Self {
-        assert_eq!(input.len(), 16);
-        let x = core::mem::transmute(core::ptr::read(input as *const _ as *const [u8; 16]));
+        let x = u32x4_generic::read_from(input).unwrap();
         dmap(x, |x| x.to_le())
     }
     #[inline(always)]
     unsafe fn unsafe_read_be(input: &[u8]) -> Self {
-        assert_eq!(input.len(), 16);
-        let x = core::mem::transmute(core::ptr::read(input as *const _ as *const [u8; 16]));
+        let x = u32x4_generic::read_from(input).unwrap();
         dmap(x, |x| x.to_be())
     }
     #[inline(always)]
     fn write_le(self, out: &mut [u8]) {
-        assert_eq!(out.len(), 16);
         let x = dmap(self, |x| x.to_le());
-        unsafe { core::ptr::write(out as *mut _ as *mut [u8; 16], core::mem::transmute(x)) }
+        x.write_to(out).unwrap();
     }
     #[inline(always)]
     fn write_be(self, out: &mut [u8]) {
-        assert_eq!(out.len(), 16);
         let x = dmap(self, |x| x.to_be());
-        unsafe { core::ptr::write(out as *mut _ as *mut [u8; 16], core::mem::transmute(x)) }
+        x.write_to(out).unwrap();
     }
 }
 impl StoreBytes for u64x2_generic {
     #[inline(always)]
     unsafe fn unsafe_read_le(input: &[u8]) -> Self {
-        assert_eq!(input.len(), 16);
-        let x = core::mem::transmute(core::ptr::read(input as *const _ as *const [u8; 16]));
+        let x = u64x2_generic::read_from(input).unwrap();
         qmap(x, |x| x.to_le())
     }
     #[inline(always)]
     unsafe fn unsafe_read_be(input: &[u8]) -> Self {
-        assert_eq!(input.len(), 16);
-        let x = core::mem::transmute(core::ptr::read(input as *const _ as *const [u8; 16]));
+        let x = u64x2_generic::read_from(input).unwrap();
         qmap(x, |x| x.to_be())
     }
     #[inline(always)]
     fn write_le(self, out: &mut [u8]) {
-        assert_eq!(out.len(), 16);
         let x = qmap(self, |x| x.to_le());
-        unsafe { core::ptr::write(out as *mut _ as *mut [u8; 16], core::mem::transmute(x)) }
+        x.write_to(out).unwrap();
     }
     #[inline(always)]
     fn write_be(self, out: &mut [u8]) {
-        assert_eq!(out.len(), 16);
         let x = qmap(self, |x| x.to_be());
-        unsafe { core::ptr::write(out as *mut _ as *mut [u8; 16], core::mem::transmute(x)) }
+        x.write_to(out).unwrap();
     }
 }
 
